#!/usr/bin/node

// 読み込むライブラリ、コンポーネントを定義します
const assert = require("assert")
const path = require("path")
var bp = require("./build/Release/btrieveJavascript")


// 1万件のデータを作成する際にループで使用します
min_x = 1
max_x = 10000

// 作成するデータファイルと1レコードの長さを定義します
btrieveFileName = "/usr/local/actianzen/data/test_cr_ins_read.btr"
recordLength = 10

// BtrieveClient はファイルの作成や削除、ファイルのオープンクローズといったエンジン全体に関する操作を行うためのクラスです。
btrieveClient = new bp.BtrieveClient()
assert(btrieveClient != null)


// キーセグメント情報を用意します
btrieveKeySegment = new bp.BtrieveKeySegment()
assert(btrieveKeySegment != null)
// データファイルの最初の8バイトをAUTOTIMESTAMP型に設定。自動的に現在のタイムスタンプが記録されます。形式は 8 バイトの符号なし整数の、Unix エポックからのナノ秒数です。
rc = btrieveKeySegment.SetField(0, 8, bp.Btrieve.DATA_TYPE_AUTOTIMESTAMP)
assert(rc == bp.Btrieve.STATUS_CODE_NO_ERROR)


//作成したキーセグメント情報をインデックス属性に登録（追加）します
btrieveIndexAttributes = new bp.BtrieveIndexAttributes()
assert(btrieveIndexAttributes != null)

rc = btrieveIndexAttributes.AddKeySegment(btrieveKeySegment)
assert(rc == bp.Btrieve.STATUS_CODE_NO_ERROR)


//ファイル属性のオブジェクトを作成し、レコード長を指定します
btrieveFileAttributes = new bp.BtrieveFileAttributes()
assert(btrieveFileAttributes != null)

rc = btrieveFileAttributes.SetFixedRecordLength(recordLength)
assert(rc == bp.Btrieve.STATUS_CODE_NO_ERROR)


// 設定した情報に基づいた新規の Btrieve ファイルを作成します
rc = btrieveClient.FileCreate(btrieveFileAttributes, btrieveIndexAttributes, btrieveFileName, bp.Btrieve.CREATE_MODE_OVERWRITE)
console.log("FileCreate rc = " + rc)
assert(rc == bp.Btrieve.STATUS_CODE_NO_ERROR)


// ファイルをオープンします
// BtrieveFileオブジェクトは、ファイルを扱うクラスです
btrieveFile = new bp.BtrieveFile()
assert(btrieveFile != null)

rc = btrieveClient.FileOpen(btrieveFile, btrieveFileName, "", bp.Btrieve.OPEN_MODE_NORMAL)
assert(rc == bp.Btrieve.STATUS_CODE_NO_ERROR)


// 1万件のサンプル時系列のデータを挿入
for (i = min_x; i <= max_x; i++)
{

    //レコード長に合わせたバッファを用意します
    record = new Buffer.alloc(recordLength)
    	
    // 例えば、Btrieve ファイルの 9～10 バイトにセンサーの値（ここでは便宜上インクリメントした数値）を書き込みます。
    record.writeUInt16LE(i, 8)
    rc = btrieveFile.RecordCreate(record.buffer)
    assert(rc == bp.Btrieve.STATUS_CODE_NO_ERROR)

    // 後で検索条件として使用するため、200番目のレコードを挿入した時間を取得
    if (i == 200) {
		record = new Buffer.alloc(recordLength)
		rc = btrieveFile.RecordRetrieveLast(bp.Btrieve.INDEX_1, record.buffer)
		time_to_search = record.readBigUInt64LE(0)
    }

}

// 保存データから最後に記録した値を取得する 
record = new Buffer.alloc(recordLength)

rc = btrieveFile.RecordRetrieveLast(bp.Btrieve.INDEX_1, record.buffer)
assert(rc >= recordLength)

// 9～10バイト目の数値データを読み取る
i = record.readUInt16LE(8)
console.log("最後に記録した値：" + i)

// 200番目のデータを保存した際に記録した時間を使って一致するレコードを取得する
key = new Buffer.alloc(8)

// キーバッファに保存しておいた時間を設定し、レコードを読み取る
key.writeBigUInt64LE(time_to_search, 0)
rc = btrieveFile.RecordRetrieve(bp.Btrieve.COMPARISON_EQUAL, bp.Btrieve.INDEX_1, key.buffer, record.buffer)

// 取り出す数値は「200」が格納されているはずです
i = record.readUInt16LE(8)
console.log("指定時間のデータ数値：" + i)

// ファイルをクローズして終了します
rc = btrieveClient.FileClose(btrieveFile)
assert(rc == bp.Btrieve.STATUS_CODE_NO_ERROR)

// 作成したファイルを削除します
//rc = btrieveClient.FileDelete(btrieveFileName)
//assert(rc == bp.Btrieve.STATUS_CODE_NO_ERROR)
